mod hyper_client_net;
use hyper_client_net::net_request;

use lox::IssuerPubKey;
use lox::OPENINV_LENGTH;
use lox::proto::*;
use serde::{Serialize, Deserialize};
use serde_with::serde_as;
use std::time::Duration;

// From https://gitlab.torproject.org/onyinyang/lox-server/-/blob/main/src/main.rs
// TODO: Move this to main Lox library?
#[serde_as]
#[derive(Serialize, Deserialize)]
pub struct Invite {
    #[serde_as(as = "[_; OPENINV_LENGTH]")]
    invite: [u8; OPENINV_LENGTH],
}

/// Get today's (real or simulated) date
///
/// This function is modified from the lox lib.rs
fn today(time_offset: Duration) -> u32 {
    // We will not encounter negative Julian dates (~6700 years ago)
    // or ones larger than 32 bits
    (time::OffsetDateTime::now_utc().date() + time_offset)
        .julian_day()
        .try_into()
        .unwrap()
}

// Download Lox Auth pubkeys
pub async fn get_lox_auth_keys(server_addr: String) -> Vec<IssuerPubKey> {
    let lox_auth_pubkeys_resp = net_request(format!("{}/pubkeys", server_addr), [].to_vec()).await;
    let lox_auth_pubkeys: Vec<IssuerPubKey> = serde_json::from_slice(&lox_auth_pubkeys_resp).unwrap();
    lox_auth_pubkeys
}

// Get an open invitation
pub async fn get_open_invitation(server_addr: String) -> [u8; OPENINV_LENGTH] {
    let open_invite_resp = net_request(format!("{}/invite", server_addr), [].to_vec()).await;
    let open_invite: [u8; OPENINV_LENGTH]  = serde_json::from_slice::<Invite>(&open_invite_resp).unwrap().invite;
    open_invite
}

// Get a Lox Credential from an open invitation
pub async fn get_lox_credential(server_addr: String, open_invite: [u8; OPENINV_LENGTH], lox_pub: IssuerPubKey) -> lox::cred::Lox {
    let (open_inv_req, state) = open_invite::request(&open_invite);
    let encoded_open_inv_req: Vec<u8> = serde_json::to_vec(&open_inv_req).unwrap();
    let encoded_open_inv_resp = net_request(format!("{}/openreq", server_addr), encoded_open_inv_req).await;
    let decoded_open_inv_resp: open_invite::Response = serde_json::from_slice(&encoded_open_inv_resp).unwrap();
    let (cred, bridgeline) = open_invite::handle_response(state, decoded_open_inv_resp, &lox_pub).unwrap();
    cred
    // TODO: Also return the bridgeline
}

// Get a migration credential to migrate to higher trust
pub async fn trust_promotion(server_addr: String, lox_cred: lox::cred::Lox, lox_pub: IssuerPubKey) -> lox::cred::Migration {
    let (prom_req, state) = trust_promotion::request(&lox_cred, &lox_pub, today(Duration::ZERO)).unwrap();
    let encoded_prom_req: Vec<u8> = serde_json::to_vec(&prom_req).unwrap();
    let encoded_prom_resp = net_request(format!("{}/promreq", server_addr), encoded_prom_req).await;
    let decoded_prom_resp: trust_promotion::Response = serde_json::from_slice(&encoded_prom_resp).unwrap();
    let migration_cred = trust_promotion::handle_response(state, decoded_prom_resp).unwrap();
    migration_cred
}
